@@ -6,11 +6,11 @@ from python_framework.api.src.converter.static import ConverterStatic
 
 
 DEFAUTL_MUTE_LOGS = False
-DEFAUTL_DISABLE = False
+DEFAUTL_ENABLED = True
 
 
 @Function
-def Emitter(*emitterArgs, manager=None, managerClient=None, disable=DEFAUTL_DISABLE, muteLogs=DEFAUTL_MUTE_LOGS, **emitterKwargs) :
+def Emitter(*emitterArgs, manager=None, managerClient=None, enabled=DEFAUTL_ENABLED, muteLogs=DEFAUTL_MUTE_LOGS, **emitterKwargs) :
     emitterManager = manager
     emitterManagerClient = managerClient
     def Wrapper(OuterClass, *args, **kwargs):
@@ -24,8 +24,7 @@ def Emitter(*emitterArgs, manager=None, managerClient=None, disable=DEFAUTL_DISA
                 self.managerClient = emitterManagerClient if not isinstance(emitterManagerClient, str) else ReflectionHelper.getAttributeOrMethodByNamePath(apiInstance, emitterManagerClient)
                 self.globals = apiInstance.globals
                 self.service = apiInstance.resource.service
-                self.enabled = self.globals.getApiSetting(ConfigurationKeyConstant.API_EMITTER_ENABLE)
-                self.disabled = disable
+                self.enabled = enabled and self.globals.getApiSetting(ConfigurationKeyConstant.API_EMITTER_ENABLE)
                 self.muteLogs = muteLogs or ConverterStatic.getValueOrDefault(self.globals.getApiSetting(ConfigurationKeyConstant.API_EMITTER_MUTE_LOGS), DEFAUTL_MUTE_LOGS and muteLogs)
         ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
@@ -37,11 +36,11 @@ def EmitterMethod(
     *methodArgs,
     requestClass = None,
     interceptor = FlaskManager.defaultResourceInterceptor,
-    disable = DEFAUTL_DISABLE,
+    enabled = DEFAUTL_ENABLED,
     muteLogs = DEFAUTL_MUTE_LOGS,
     **methodKwargs
 ):
-    resourceMethodDisable = disable
+    resourceMethodEnabled = enabled
     resourceMethodMuteLogs = muteLogs
     resourceInterceptor = interceptor
     def innerMethodWrapper(resourceMethod, *innerMethodArgs, **innerMethodKwargs) :
@@ -49,7 +48,8 @@ def EmitterMethod(
         apiInstance = FlaskManager.getApi()
         methodClassName = ReflectionHelper.getMethodClassName(resourceMethod)
         methodName = ReflectionHelper.getName(resourceMethod)
-        resourceMethod.disabled = disable
+        resourceTypeIsEnabled = apiInstance.globals.getApiSetting(ConfigurationKeyConstant.API_EMITTER_ENABLE)
+        resourceMethod.enabled = resourceMethodEnabled and resourceTypeIsEnabled
         resourceMethod.id = methodKwargs.get('id', f'{methodClassName}{c.DOT}{methodName}')
         resourceMethod.muteLogs = resourceMethodMuteLogs or ConverterStatic.getValueOrDefault(apiInstance.globals.getApiSetting(ConfigurationKeyConstant.API_EMITTER_MUTE_LOGS), DEFAUTL_MUTE_LOGS and resourceMethodMuteLogs)
         emitterArgs = [*methodArgs]
@@ -62,7 +62,7 @@ def EmitterMethod(
             resourceInstance = FlaskManager.getResourceSelf(apiInstance, FlaskManager.KW_EMITTER_RESOURCE, resourceInstanceName)
             args = FlaskManager.getArgumentInFrontOfArgs(args, resourceInstance) ###- resourceInstance = args[0]
             muteLogs = resourceInstance.muteLogs or resourceMethod.muteLogs
-            if resourceInstance.enabled and not resourceInstance.disabled and not resourceMethod.disabled:
+            if resourceInstance.enabled and resourceMethod.enabled:
                 if not muteLogs:
                     log.info(resourceMethod, f'{resourceMethod.id} emitter method started with args={methodArgs} and kwargs={methodKwargs}')
                 methodReturn = None
@@ -76,10 +76,11 @@ def EmitterMethod(
                 if not muteLogs:
                     log.info(resourceMethod, f'{resourceMethod.id} emitter method finished')
                 return methodReturn
-            if not muteLogs:
-                log.warning(resourceMethod, f'{resourceMethod.id} emitter method didn{c.SINGLE_QUOTE}t started. {"Handlers are disabled" if not resourceInstance.enabled else "This emitter class is disabled" if resourceInstance.disabled else "This emitter method is disabled"}')
+            else:
+                if not muteLogs:
+                    log.warning(resourceMethod, f'{resourceMethod.id} emitter method didn{c.SINGLE_QUOTE}t started. {"Handlers are disabled" if resourceTypeIsEnabled else "This emitter class is disabled" if not resourceInstance.enabled else "This emitter method is disabled"}')
         ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceMethod)
-        innerResourceInstanceMethod.disable = resourceMethodDisable
+        innerResourceInstanceMethod.enabled = resourceMethodEnabled
         innerResourceInstanceMethod.muteLogs = resourceMethodMuteLogs
         innerResourceInstanceMethod.interceptor = resourceInterceptor
         return innerResourceInstanceMethod
