@@ -1,48 +1,17 @@
 import requests
 from python_helper import Constant as c
-from python_helper import ReflectionHelper, ObjectHelper, log, Function, StringHelper
+from python_helper import ReflectionHelper, ObjectHelper, log, Function
 
+from python_framework.api.src.constant import HttpClientConstant, LogConstant
+from python_framework.api.src.domain import HttpDomain
 from python_framework.api.src.util import FlaskUtil
 from python_framework.api.src.util import Serializer
-from python_framework.api.src.domain import HttpDomain
-from python_framework.api.src.constant import HttpClientConstant, LogConstant
 from python_framework.api.src.converter.static import ConverterStatic
 from python_framework.api.src.enumeration.HttpStatus import HttpStatus
 from python_framework.api.src.service.flask import FlaskManager
 from python_framework.api.src.service.openapi import OpenApiManager
-from python_framework.api.src.service.ExceptionHandler import GlobalException
-
-
-class HttpClientEvent(Exception):
-    def __init__(self, verb, *args, **kwargs):
-        if ObjectHelper.isNone(verb):
-            raise Exception('Http client event verb cannot be none')
-        Exception.__init__(self, f'Http client {verb} event')
-        self.verb = verb
-        self.args = args
-        self.kwargs = kwargs
-
-
-class ManualHttpClientEvent(Exception):
-    def __init__(self, completeResponse):
-        Exception.__init__(self, f'Manual http client event')
-        self.completeResponse = completeResponse
-
-
-def getHttpClientEvent(resourceInstanceMethod, *args, **kwargs):
-    completeResponse = None
-    try:
-        completeResponse = resourceInstanceMethod(*args, **kwargs)
-    except HttpClientEvent as httpClientEvent:
-        return httpClientEvent
-    except Exception as exception:
-        raise exception
-    if ObjectHelper.isNotNone(completeResponse):
-        return ManualHttpClientEvent(completeResponse)
-
-
-def raiseHttpClientEventNotFoundException(*args, **kwargs):
-    raise Exception('HttpClientEvent not found')
+from python_framework.api.src.util import ClientUtil
+from python_framework.api.src.util.ClientUtil import HttpClientEvent, ManualHttpClientEvent
 
 
 @Function
@@ -166,7 +135,7 @@ def HttpClientMethod(
             **kwargs
         ):
             verb = HttpDomain.Verb.OPTIONS
-            url, params, headers, body, timeout, logRequest = parseParameters(
+            url, params, headers, body, timeout, logRequest = ClientUtil.parseParameters(
                 resourceInstance,
                 clientMethodConfig,
                 additionalUrl,
@@ -198,7 +167,7 @@ def HttpClientMethod(
             **kwargs
         ):
             verb = HttpDomain.Verb.GET
-            url, params, headers, body, timeout, logRequest = parseParameters(
+            url, params, headers, body, timeout, logRequest = ClientUtil.parseParameters(
                 resourceInstance,
                 clientMethodConfig,
                 additionalUrl,
@@ -230,7 +199,7 @@ def HttpClientMethod(
             **kwargs
         ):
             verb = HttpDomain.Verb.POST
-            url, params, headers, body, timeout, logRequest = parseParameters(
+            url, params, headers, body, timeout, logRequest = ClientUtil.parseParameters(
                 resourceInstance,
                 clientMethodConfig,
                 additionalUrl,
@@ -262,7 +231,7 @@ def HttpClientMethod(
             **kwargs
         ):
             verb = HttpDomain.Verb.PUT
-            url, params, headers, body, timeout, logRequest = parseParameters(
+            url, params, headers, body, timeout, logRequest = ClientUtil.parseParameters(
                 resourceInstance,
                 clientMethodConfig,
                 additionalUrl,
@@ -294,7 +263,7 @@ def HttpClientMethod(
             **kwargs
         ):
             verb = HttpDomain.Verb.PATCH
-            url, params, headers, body, timeout, logRequest = parseParameters(
+            url, params, headers, body, timeout, logRequest = ClientUtil.parseParameters(
                 resourceInstance,
                 clientMethodConfig,
                 additionalUrl,
@@ -326,7 +295,7 @@ def HttpClientMethod(
             **kwargs
         ):
             verb = HttpDomain.Verb.DELETE
-            url, params, headers, body, timeout, logRequest = parseParameters(
+            url, params, headers, body, timeout, logRequest = ClientUtil.parseParameters(
                 resourceInstance,
                 clientMethodConfig,
                 additionalUrl,
@@ -389,23 +358,23 @@ def HttpClientMethod(
                 )
                 FlaskManager.validateArgs(args, requestClass, resourceInstanceMethod)
                 clientResponse = None
-                httpClientEvent = getHttpClientEvent(resourceInstanceMethod, *args, **kwargs)
+                httpClientEvent = ClientUtil.getHttpClientEvent(resourceInstanceMethod, *args, **kwargs)
                 if isinstance(httpClientEvent, ManualHttpClientEvent):
                     completeResponse = httpClientEvent.completeResponse
                 elif isinstance(httpClientEvent, HttpClientEvent):
                     try :
                         clientResponse = HTTP_CLIENT_RESOLVERS_MAP.get(
                             httpClientEvent.verb,
-                            raiseHttpClientEventNotFoundException
+                            ClientUtil.raiseHttpClientEventNotFoundException
                         )(
                             resourceInstance,
                             *httpClientEvent.args,
                             **httpClientEvent.kwargs
                         )
                     except Exception as exception:
-                        raiseException(clientResponse, exception)
-                    raiseExceptionIfNeeded(clientResponse)
-                    completeResponse = getCompleteResponse(clientResponse, responseClass, produces)
+                        ClientUtil.raiseException(clientResponse, exception)
+                    ClientUtil.raiseExceptionIfNeeded(clientResponse)
+                    completeResponse = ClientUtil.getCompleteResponse(clientResponse, responseClass, produces)
                     FlaskManager.validateCompleteResponse(responseClass, completeResponse)
                 else:
                     raise Exception('Unknown http client event')
@@ -450,158 +419,3 @@ def HttpClientMethod(
 
         return innerResourceInstanceMethod
     return innerMethodWrapper
-
-
-@Function
-def getUrl(client, clientMethodConfig, additionalUrl):
-    return StringHelper.join(
-        [
-            ConverterStatic.getValueOrDefault(u, c.BLANK) for u in [
-                client.url,
-                clientMethodConfig.url,
-                additionalUrl
-            ]
-        ],
-        character = c.BLANK
-    )
-
-
-@Function
-def getHeaders(client, clientMethodConfig, headers):
-    return {
-        **ConverterStatic.getValueOrDefault(client.headers, dict()),
-        **{HttpDomain.HeaderKey.CONTENT_TYPE: clientMethodConfig.consumes},
-        **ConverterStatic.getValueOrDefault(clientMethodConfig.headers, dict()),
-        **ConverterStatic.getValueOrDefault(headers, dict())
-    }
-
-
-@Function
-def getTimeout(client, clientMethodConfig, timeout):
-    return ConverterStatic.getValueOrDefault(timeout, ConverterStatic.getValueOrDefault(clientMethodConfig.timeout, client.timeout))
-
-
-@Function
-def getLogRequest(client, clientMethodConfig, logRequest):
-    return client.logRequest or clientMethodConfig.logRequest or logRequest
-
-
-@Function
-def parseParameters(client, clientMethodConfig, additionalUrl, params, headers, body, timeout, logRequest):
-    url = getUrl(client, clientMethodConfig, additionalUrl)
-    params = ConverterStatic.getValueOrDefault(params, dict())
-    headers = getHeaders(client, clientMethodConfig, headers)
-    body = ConverterStatic.getValueOrDefault(body, dict())
-    timeout = getTimeout(client, clientMethodConfig, timeout)
-    logRequest = getLogRequest(client, clientMethodConfig, logRequest)
-    return url, params, headers, body, timeout, logRequest
-
-
-def raiseException(clientResponse, exception):
-    raise GlobalException(
-        logMessage = getErrorMessage(clientResponse, exception=exception),
-        url = FlaskUtil.safellyGetRequestUrlFromResponse(clientResponse),
-        status = FlaskUtil.safellyGetResponseStatus(clientResponse),
-        logHeaders = {
-            HttpClientConstant.REQUEST_HEADERS_KEY: FlaskUtil.safellyGetRequestHeadersFromResponse(clientResponse),
-            HttpClientConstant.RESPONSE_HEADERS_KEY: FlaskUtil.safellyGetResponseHeaders(clientResponse)
-        },
-        logPayload = {
-            HttpClientConstant.REQUEST_BODY_KEY: FlaskUtil.safellyGetRequestJsonFromResponse(clientResponse),
-            HttpClientConstant.RESPONSE_BODY_KEY: FlaskUtil.safellyGetResponseJson(clientResponse)
-        },
-        context = HttpDomain.CLIENT_CONTEXT
-    )
-
-
-def raiseExceptionIfNeeded(clientResponse):
-    status = FlaskUtil.safellyGetResponseStatus(clientResponse) ###- clientResponse.status_code
-    if ObjectHelper.isNone(status) or 500 <= status:
-        raise GlobalException(
-            logMessage = getErrorMessage(clientResponse),
-            url = FlaskUtil.safellyGetRequestUrlFromResponse(clientResponse),
-            status = status,
-            logHeaders = {
-                HttpClientConstant.REQUEST_HEADERS_KEY: FlaskUtil.safellyGetRequestHeadersFromResponse(clientResponse),
-                HttpClientConstant.RESPONSE_HEADERS_KEY: FlaskUtil.safellyGetResponseHeaders(clientResponse)
-            },
-            logPayload = {
-                HttpClientConstant.REQUEST_BODY_KEY: FlaskUtil.safellyGetRequestJsonFromResponse(clientResponse),
-                HttpClientConstant.RESPONSE_BODY_KEY: FlaskUtil.safellyGetResponseJson(clientResponse)
-            },
-            context = HttpDomain.CLIENT_CONTEXT
-        )
-    elif 400 <= status:
-        raise GlobalException(
-            message = getErrorMessage(clientResponse),
-            logMessage = HttpClientConstant.ERROR_AT_CLIENT_CALL_MESSAGE,
-            url = FlaskUtil.safellyGetRequestUrlFromResponse(clientResponse),
-            status = status,
-            logHeaders = {
-                HttpClientConstant.REQUEST_HEADERS_KEY: FlaskUtil.safellyGetRequestHeadersFromResponse(clientResponse),
-                HttpClientConstant.RESPONSE_HEADERS_KEY: FlaskUtil.safellyGetResponseHeaders(clientResponse)
-            },
-            logPayload = {
-                HttpClientConstant.REQUEST_BODY_KEY: FlaskUtil.safellyGetRequestJsonFromResponse(clientResponse),
-                HttpClientConstant.RESPONSE_BODY_KEY: FlaskUtil.safellyGetResponseJson(clientResponse)
-            },
-            context = HttpDomain.CLIENT_CONTEXT
-        )
-
-
-@Function
-def getCompleteResponse(clientResponse, responseClass, produces, fallbackStatus=HttpStatus.INTERNAL_SERVER_ERROR):
-    responseBody, responseHeaders, responseStatus = dict(), dict(), fallbackStatus
-    responseHeaders = FlaskUtil.safellyGetResponseHeaders(clientResponse)
-    responseBody = FlaskUtil.safellyGetResponseJson(clientResponse)
-    try :
-        responseStatus = HttpStatus.map(HttpStatus.NOT_FOUND if ObjectHelper.isNone(clientResponse.status_code) else clientResponse.status_code)
-    except Exception as exception :
-        responseStatus = HttpStatus.map(fallbackStatus)
-        log.warning(getCompleteResponse, f'Not possible to get client response status. Returning {responseStatus} by default', exception=exception)
-    responseHeaders = {
-        **{HttpDomain.HeaderKey.CONTENT_TYPE: produces},
-        **responseHeaders
-    }
-    responseStatus = ConverterStatic.getValueOrDefault(responseStatus, HttpStatus.map(fallbackStatus))
-    if ObjectHelper.isNotNone(responseClass):
-        responseBody = Serializer.convertFromJsonToObject(responseBody, responseClass)
-    return responseBody, responseHeaders, responseStatus
-
-
-@Function
-def getErrorMessage(clientResponse, exception=None):
-    bodyAsJson = getClientResponseBodyAsJson(clientResponse)
-    completeErrorMessage = f'{HttpClientConstant.ERROR_AT_CLIENT_CALL_MESSAGE}{c.DOT_SPACE}{HttpClientConstant.CLIENT_DID_NOT_SENT_ANY_MESSAGE}'
-    errorMessage = HttpClientConstant.CLIENT_DID_NOT_SENT_ANY_MESSAGE
-    possibleErrorMessage = None
-    try:
-        if ObjectHelper.isNotNone(clientResponse):
-            if ObjectHelper.isDictionary(bodyAsJson):
-                possibleErrorMessage = getErrorMessageFromClientResponseBodyAsJson(bodyAsJson)
-            if ObjectHelper.isList(bodyAsJson) and 0 < len(bodyAsJson):
-                possibleErrorMessage = getErrorMessageFromClientResponseBodyAsJson(bodyAsJson[0])
-        if ObjectHelper.isNotNone(possibleErrorMessage) and StringHelper.isNotBlank(possibleErrorMessage):
-            errorMessage = f'{c.LOG_CAUSE}{possibleErrorMessage}'
-        else:
-            log.debug(getErrorMessage, f'Client response {FlaskUtil.safellyGetResponseJson(clientResponse)}')
-        exceptionPortion = HttpClientConstant.ERROR_AT_CLIENT_CALL_MESSAGE if ObjectHelper.isNone(exception) or StringHelper.isBlank(exception) else str(exception)
-        completeErrorMessage = f'{exceptionPortion}{c.DOT_SPACE}{errorMessage}'
-    except Exception as innerException:
-        log.warning(getErrorMessage, f'Not possible to get error message. Returning "{completeErrorMessage}" by default', exception=innerException)
-    return completeErrorMessage
-
-
-def getClientResponseBodyAsJson(clientResponse):
-    bodyAsJson = {}
-    try :
-        bodyAsJson = clientResponse.json()
-    except Exception as exception :
-        bodyAsJsonException = FlaskUtil.safellyGetResponseJson(clientResponse)
-        log.log(getErrorMessage, f'Invalid client response: {bodyAsJsonException}', exception=exception)
-        log.debug(getErrorMessage, f'Not possible to get error message from client response: {bodyAsJsonException}. Proceeding with value {bodyAsJson} by default', exception=exception, muteStackTrace=True)
-    return bodyAsJson
-
-
-def getErrorMessageFromClientResponseBodyAsJson(bodyAsJson):
-    return bodyAsJson.get('message', bodyAsJson.get('error', HttpClientConstant.CLIENT_DID_NOT_SENT_ANY_MESSAGE)).strip()
